@@ -1,9 +1,9 @@
 /*
- * $Id: dialog.c,v 1.246 2016/02/09 00:05:39 tom Exp $
+ * $Id: dialog.c,v 1.259 2017/01/31 01:57:08 tom Exp $
  *
  *  cdialog - Display simple dialog boxes from shell scripts
  *
- *  Copyright 2000-2015,2016	Thomas E. Dickey
+ *  Copyright 2000-2016,2017	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -156,6 +156,7 @@ typedef enum {
 #ifdef HAVE_XDIALOG2
     ,o_buildlist
     ,o_rangebox
+    ,o_reorder
     ,o_treeview
 #endif
 #if defined(HAVE_XDIALOG2) || defined(HAVE_WHIPTAIL)
@@ -188,7 +189,8 @@ typedef struct {
     callerFn *jumper;
 } Mode;
 
-static bool *dialog_opts;
+static int known_opts = 0;
+static const char **dialog_opts;
 static char **dialog_argv;
 
 static char **special_argv = 0;
@@ -198,6 +200,15 @@ static bool ignore_unknown = FALSE;
 
 static const char *program = "dialog";
 
+#ifdef NO_LEAKS
+typedef struct _all_blobs {
+    struct _all_blobs *next;
+    void *blob;
+} AllBlobs;
+
+static AllBlobs *all_blobs;
+#endif
+
 /*
  * The options[] table is organized this way to make it simple to maintain
  * a sorted list of options for the help-message.
@@ -329,10 +340,11 @@ static const Options options[] = {
     { "iso-week",	o_iso_week,		1, NULL },
 #endif
 #ifdef HAVE_XDIALOG2
-    { "buildlist",	o_buildlist,		2, "<text> <height> <width> <tag1> <item1> <status1>..." },
+    { "buildlist",	o_buildlist,		2, "<text> <height> <width> <list-height> <tag1> <item1> <status1>..." },
     { "no-items", 	o_no_items,		1, "" },
     { "no-tags", 	o_no_tags,		1, "" },
     { "rangebox",	o_rangebox,		2, "<text> <height> <width> <min-value> <max-value> <default-value>" },
+    { "reorder", 	o_reorder,		1, "" },
     { "treeview",	o_treeview,		2, "<text> <height> <width> <list-height> <tag1> <item1> <status1> <depth1>..." },
 #endif
 #if defined(HAVE_XDIALOG2) || defined(HAVE_WHIPTAIL)
@@ -345,6 +357,73 @@ static const Options options[] = {
 };
 /* *INDENT-ON* */
 
+#ifdef NO_LEAKS
+static void
+ignore_leak(void *value)
+{
+    AllBlobs *next = dlg_calloc(AllBlobs, (size_t) 1);
+    next->blob = value;
+    next->next = all_blobs;
+    all_blobs = next;
+}
+
+static void
+handle_leaks(void)
+{
+    while (all_blobs != 0) {
+	char *blob = all_blobs->blob;
+	AllBlobs *next = all_blobs->next;
+	free(blob);
+	free(all_blobs);
+	all_blobs = next;
+    }
+    if (special_argv != 0) {
+	free(special_argv[0]);
+	free(special_argv);
+	special_argv = 0;
+	special_argc = 0;
+    }
+}
+#else
+#define handle_leaks()		/* nothing */
+#define ignore_leak(n)		/* nothing */
+#endif
+
+#define OptionChars "\
+0123456789\
+-\
+abcdefghijklmnopqrstuvwxyz\
+"
+
+/*
+ * Check if the given string from main's argv is an option.
+ */
+static bool
+isOption(const char *arg)
+{
+    bool result = FALSE;
+
+    if (arg != 0) {
+	if (dialog_opts != 0) {
+	    int n;
+	    for (n = 0; dialog_opts[n] != 0; ++n) {
+		if (dialog_opts[n] == arg) {
+		    result = TRUE;
+		    break;
+		}
+	    }
+	} else if (!strncmp(arg, "--", (size_t) 2) && isalpha(UCH(arg[2]))) {
+	    if (strlen(arg) == (strspn) (arg, OptionChars)) {
+		result = TRUE;
+	    } else {
+		handle_leaks();
+		dlg_exiterr("Invalid option \"%s\"", arg);
+	    }
+	}
+    }
+    return result;
+}
+
 /*
  * Make an array showing which argv[] entries are options.  Use "--" as a
  * special token to escape the next argument, allowing it to begin with "--".
@@ -364,28 +443,50 @@ unescape_argv(int *argcp, char ***argvp)
     int j, k;
     int limit_includes = 20 + *argcp;
     int count_includes = 0;
-    bool changed = FALSE;
     bool doalloc = FALSE;
     char *filename;
+    const char **my_argv = 0;
+    int my_argc;
 
-    dialog_opts = dlg_calloc(bool, (size_t) *argcp + 1);
-    assert_ptr(dialog_opts, "unescape_argv");
+    for (k = 0; k < 2; ++k) {
+
+	my_argc = 0;
+	if (special_argv != 0) {
+	    for (j = 0; special_argv[j] != 0; ++j) {
+		if (!strcmp(special_argv[j], "--")) {
+		    break;
+		} else if (isOption(special_argv[j])) {
+		    if (k != 0)
+			my_argv[my_argc] = special_argv[j];
+		    my_argc++;
+		}
+	    }
+	}
+
+	if (k == 0) {
+	    my_argc += (*argcp + 1);
+	    my_argv = dlg_calloc(const char *, (size_t) my_argc);
+	    assert_ptr(my_argv, "unescape_argv");
+	}
+    }
 
     for (j = 1; j < *argcp; j++) {
 	bool escaped = FALSE;
 	if (!strcmp((*argvp)[j], "--")) {
 	    escaped = TRUE;
-	    changed = dlg_eat_argv(argcp, argvp, j, 1);
+	    dlg_eat_argv(argcp, argvp, j, 1);
 	} else if (!strcmp((*argvp)[j], "--args")) {
 	    fprintf(stderr, "Showing arguments at arg%d\n", j);
 	    for (k = 0; k < *argcp; ++k) {
 		fprintf(stderr, " arg%d:%s\n", k, (*argvp)[k]);
 	    }
-	    changed = dlg_eat_argv(argcp, argvp, j, 1);
+	    dlg_eat_argv(argcp, argvp, j, 1);
 	    --j;
 	} else if (!strcmp((*argvp)[j], "--file")) {
-	    if (++count_includes > limit_includes)
+	    if (++count_includes > limit_includes) {
+		handle_leaks();
 		dlg_exiterr("Too many --file options");
+	    }
 
 	    if ((filename = (*argvp)[j + 1]) != 0) {
 		FILE *fp;
@@ -403,6 +504,7 @@ unescape_argv(int *argcp, char ***argvp)
 		}
 
 		if (fp) {
+		    DLG_TRACE(("# opened --file %s ..\n", filename));
 		    blob = NULL;
 		    length = 0;
 		    do {
@@ -413,12 +515,15 @@ unescape_argv(int *argcp, char ***argvp)
 					     (size_t) BUFSIZ,
 					   fp);
 			length += bytes_read;
-			if (ferror(fp))
+			if (ferror(fp)) {
+			    handle_leaks();
 			    dlg_exiterr("error on filehandle in unescape_argv");
+			}
 		    } while (bytes_read == BUFSIZ);
 		    fclose(fp);
 
 		    blob[length] = '\0';
+		    ignore_leak(blob);
 
 		    list = dlg_string_to_argv(blob);
 		    added = dlg_count_argv(list);
@@ -433,6 +538,7 @@ unescape_argv(int *argcp, char ***argvp)
 			    assert_ptr(*argvp, "unescape_argv");
 			} else {
 			    char **newp = dlg_malloc(char *, need);
+			    ignore_leak(newp);
 			    assert_ptr(newp, "unescape_argv");
 			    for (n = 0; n < *argcp; ++n) {
 				newp[n] = (*argvp)[n];
@@ -441,14 +547,13 @@ unescape_argv(int *argcp, char ***argvp)
 			    *argvp = newp;
 			    doalloc = TRUE;
 			}
-			dialog_opts = dlg_realloc(bool, need, dialog_opts);
-			assert_ptr(dialog_opts, "unescape_argv");
+			my_argv = dlg_realloc(const char *, need, my_argv);
+			assert_ptr(my_argv, "unescape_argv");
 
 			/* Shift the arguments after '--file <filepath>'
 			   right by (added - 2) positions */
 			for (n = *argcp - 1; n >= j + 2; --n) {
 			    (*argvp)[n + added - 2] = (*argvp)[n];
-			    dialog_opts[n + added - 2] = dialog_opts[n];
 			}
 		    } else if (added < 2) {
 			/* 0 or 1 argument read from the included file
@@ -456,24 +561,25 @@ unescape_argv(int *argcp, char ***argvp)
 			   left by (2 - added) positions */
 			for (n = j + added; n + 2 - added < *argcp; ++n) {
 			    (*argvp)[n] = (*argvp)[n + 2 - added];
-			    dialog_opts[n] = dialog_opts[n + 2 - added];
 			}
 		    }
 		    /* Copy the inserted arguments to *argvp */
 		    for (n = 0; n < added; ++n) {
 			(*argvp)[n + j] = list[n];
-			dialog_opts[n + j] = FALSE;
 		    }
 		    *argcp += added - 2;
 		    (*argvp)[*argcp] = 0;	/* Write the NULL terminator */
 		    free(list);	/* No-op if 'list' is NULL */
 		    /* Force rescan starting from the first inserted argument */
 		    --j;
+		    DLG_TRACE(("# finished --file\n"));
 		    continue;
 		} else {
+		    handle_leaks();
 		    dlg_exiterr("Cannot open --file %s", filename);
 		}
 	    } else {
+		handle_leaks();
 		dlg_exiterr("No value given for --file");
 	    }
 	}
@@ -481,54 +587,20 @@ unescape_argv(int *argcp, char ***argvp)
 	    && (*argvp)[j] != 0
 	    && !strncmp((*argvp)[j], "--", (size_t) 2)
 	    && isalpha(UCH((*argvp)[j][2]))) {
-	    dialog_opts[j] = TRUE;
+	    my_argv[my_argc++] = (*argvp)[j];
+	    DLG_TRACE(("# option argv[%d]=%s\n", j, (*argvp)[j]));
 	}
     }
 
-    /* if we didn't find any "--" tokens, there's no reason to do the table
-     * lookup in isOption()
-     */
-    if (!changed) {
-	free(dialog_opts);
-	dialog_opts = 0;
-    }
+    my_argv[my_argc] = 0;
+
+    known_opts = my_argc;
+    dialog_opts = my_argv;
+
+    DLG_TRACE(("# %d options vs %d arguments\n", known_opts, *argcp));
     dialog_argv = (*argvp);
 }
 
-#define OptionChars "\
-0123456789\
--\
-abcdefghijklmnopqrstuvwxyz\
-"
-
-/*
- * Check if the given string from main's argv is an option.
- */
-static bool
-isOption(const char *arg)
-{
-    bool result = FALSE;
-
-    if (arg != 0) {
-	if (dialog_opts != 0) {
-	    int n;
-	    for (n = 1; dialog_argv[n] != 0; ++n) {
-		if (dialog_argv[n] == arg) {
-		    result = dialog_opts[n];
-		    break;
-		}
-	    }
-	} else if (!strncmp(arg, "--", (size_t) 2) && isalpha(UCH(arg[2]))) {
-	    if (strlen(arg) == (strspn) (arg, OptionChars)) {
-		result = TRUE;
-	    } else {
-		dlg_exiterr("Invalid option \"%s\"", arg);
-	    }
-	}
-    }
-    return result;
-}
-
 static eOptions
 lookupOption(const char *name, int pass)
 {
@@ -551,6 +623,7 @@ lookupOption(const char *name, int pass)
 static void
 Usage(const char *msg)
 {
+    handle_leaks();
     dlg_exiterr("Error: %s.\nUse --help to list options.\n\n", msg);
 }
 
@@ -578,19 +651,23 @@ howmany_tags(char *argv[], int group)
 {
     int result = 0;
     int have;
-    const char *format = "Expected %d arguments, found only %d";
     char temp[80];
 
     while (argv[0] != 0) {
 	if (isOption(argv[0]))
 	    break;
 	if ((have = arg_rest(argv)) < group) {
+	    const char *format = _("Expected %d arguments, found only %d");
 	    sprintf(temp, format, group, have);
 	    Usage(temp);
+	} else if ((have % group) != 0) {
+	    const char *format = _("Expected %d arguments, found extra %d");
+	    sprintf(temp, format, group, (have % group));
+	    Usage(temp);
 	}
 
-	argv += group;
-	result++;
+	argv += have;
+	result += (have / group);
     }
 
     return result;
@@ -897,7 +974,7 @@ call_buildlist(CALLARGS)
 			      numeric_arg(av, 3),
 			      numeric_arg(av, 4),
 			      tags, av + 5,
-			      TRUE);
+			      dialog_vars.reorder);
     RestoreNoTags();
     return result;
 }
@@ -1301,7 +1378,7 @@ Help(void)
     static const char *const tbl_1[] =
     {
 	"cdialog (ComeOn Dialog!) version %s",
-	"Copyright 2000-2015,2016 Thomas E. Dickey",
+	"Copyright 2000-2016,2017 Thomas E. Dickey",
 	"This is free software; see the source for copying conditions.  There is NO",
 	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.",
 	"",
@@ -1362,6 +1439,7 @@ Help(void)
     PrintList(tbl_3);
 
     free(opts);
+    handle_leaks();
     dlg_exit(DLG_EXIT_OK);
 }
 
@@ -1380,13 +1458,13 @@ process_trace_option(char **argv, int *offset)
     if (dialog_state.trace_output == 0) {
 	dlg_trace(optionString(argv, offset));
     } else {
-	dlg_trace_msg("# ignore extra --trace option\n");
+	DLG_TRACE(("# ignore extra --trace option\n"));
 	*offset += 1;
     }
 
-    dlg_trace_msg("# Parameters:\n");
+    DLG_TRACE(("# Parameters:\n"));
     for (j = 0; argv[j] != 0; ++j) {
-	dlg_trace_msg("# argv[%d] = %s\n", j, argv[j]);
+	DLG_TRACE(("# argv[%d] = %s\n", j, argv[j]));
     }
 }
 #endif
@@ -1401,10 +1479,10 @@ process_common_options(int argc, char **argv, int offset, bool output)
 {
     bool done = FALSE;
 
-    dlg_trace_msg("# process_common_options, offset %d\n", offset);
+    DLG_TRACE(("# process_common_options, offset %d\n", offset));
 
     while (offset < argc && !done) {	/* Common options */
-	dlg_trace_msg("#\targv[%d] = %s\n", offset, argv[offset]);
+	DLG_TRACE(("#\targv[%d] = %s\n", offset, argv[offset]));
 	switch (lookupOption(argv[offset], 1)) {
 	case o_title:
 	    dialog_vars.title = optionString(argv, &offset);
@@ -1658,6 +1736,9 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	    break;
 #endif
 #ifdef HAVE_XDIALOG
+	case o_reorder:
+	    dialog_vars.reorder = TRUE;
+	    break;
 	case o_week_start:
 	    dialog_vars.week_start = optionString(argv, &offset);
 	    break;
@@ -1685,7 +1766,7 @@ init_result(char *buffer)
 {
     static bool first = TRUE;
 
-    dlg_trace_msg("# init_result\n");
+    DLG_TRACE(("# init_result\n"));
 
     /* clear everything we do not save for the next widget */
     memset(&dialog_vars, 0, sizeof(dialog_vars));
@@ -1741,6 +1822,7 @@ main(int argc, char *argv[])
     (void) setlocale(LC_ALL, "");
 #endif
 
+    init_result(my_buffer);	/* honor $DIALOGOPTS */
     unescape_argv(&argc, &argv);
     program = argv[0];
     dialog_state.output = stderr;
@@ -1766,13 +1848,17 @@ main(int argc, char *argv[])
 	    break;
 	case o_input_fd:
 	    if ((j = optionValue(argv, &offset)) < 0
-		|| (dialog_state.input = fdopen(j, "r")) == 0)
+		|| (dialog_state.input = fdopen(j, "r")) == 0) {
+		handle_leaks();
 		dlg_exiterr("Cannot open input-fd\n");
+	    }
 	    break;
 	case o_output_fd:
 	    if ((j = optionValue(argv, &offset)) < 0
-		|| (dialog_state.output = fdopen(j, "w")) == 0)
+		|| (dialog_state.output = fdopen(j, "w")) == 0) {
+		handle_leaks();
 		dlg_exiterr("Cannot open output-fd\n");
+	    }
 	    break;
 	case o_keep_tite:
 	    keep_tite = TRUE;
@@ -1804,13 +1890,11 @@ main(int argc, char *argv[])
 	    ++offset;
 	    continue;
 	}
-	dlg_trace_msg("# discarding %d parameters starting with argv[%d] (%s)\n",
-		      1 + offset - base, base,
-		      argv[base]);
+	DLG_TRACE(("# discarding %d parameters starting with argv[%d] (%s)\n",
+		   1 + offset - base, base,
+		   argv[base]));
 	for (j = base; j < argc; ++j) {
 	    dialog_argv[j] = dialog_argv[j + 1 + (offset - base)];
-	    if (dialog_opts != 0)
-		dialog_opts[j] = dialog_opts[j + 1 + (offset - base)];
 	}
 	argc -= (1 + offset - base);
 	offset = base;
@@ -1856,8 +1940,10 @@ main(int argc, char *argv[])
 	    sprintf(temp, "Expected a filename for %.50s", argv[1]);
 	    Usage(temp);
 	}
-	if (dlg_parse_rc() == -1)	/* Read the configuration file */
+	if (dlg_parse_rc() == -1) {	/* Read the configuration file */
+	    handle_leaks();
 	    dlg_exiterr("dialog: dlg_parse_rc");
+	}
 	dlg_create_rc(argv[2]);
 	return DLG_EXIT_OK;
     }
@@ -1951,6 +2037,7 @@ main(int argc, char *argv[])
 		    } else {
 			argv[j] = strdup("?");
 		    }
+		    ignore_leak(argv[j]);
 		}
 		break;
 	    }
@@ -1959,7 +2046,7 @@ main(int argc, char *argv[])
 	retval = show_result((*(modePtr->jumper)) (dialog_vars.title,
 						   argv + offset,
 						   &offset_add));
-	dlg_trace_msg("# widget returns %d\n", retval);
+	DLG_TRACE(("# widget returns %d\n", retval));
 	offset += offset_add;
 
 	if (dialog_vars.input_result != my_buffer) {
@@ -2006,13 +2093,6 @@ main(int argc, char *argv[])
 	(void) refresh();
 	end_dialog();
     }
-#ifdef NO_LEAKS
-    if (special_argv != 0) {
-	free(special_argv[0]);
-	free(special_argv);
-	special_argv = 0;
-	special_argc = 0;
-    }
-#endif
+    handle_leaks();
     dlg_exit(retval);
 }
