@@ -1,9 +1,9 @@
 /*
- *  $Id: dlg_keys.c,v 1.36 2016/02/03 01:59:12 tom Exp $
+ *  $Id: dlg_keys.c,v 1.42 2017/01/31 00:26:04 tom Exp $
  *
  *  dlg_keys.c -- runtime binding support for dialog
  *
- *  Copyright 2006-2014,2016 Thomas E. Dickey
+ *  Copyright 2006-2016,2017 Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -26,6 +26,10 @@
 
 #define LIST_BINDINGS struct _list_bindings
 
+#define CHR_BACKSLASH   '\\'
+#define IsOctal(ch)     ((ch) >= '0' && (ch) <= '7')
+#define TableSize(name) (sizeof(name)/sizeof(name[0]))
+
 LIST_BINDINGS {
     LIST_BINDINGS *link;
     WINDOW *win;		/* window on which widget gets input */
@@ -70,7 +74,7 @@ dlg_register_window(WINDOW *win, const char *name, DLG_KEYS_BINDING * binding)
      * registered, there is no other way to see what bindings are available,
      * than by running dialog and tracing it.
      */
-    dlg_trace_msg("# dlg_register_window %s\n", name);
+    DLG_TRACE(("# dlg_register_window %s\n", name));
     dlg_dump_window_keys(dialog_state.trace_output, win);
 #endif
 }
@@ -320,7 +324,7 @@ typedef struct {
 
 #define ASCII_NAME(name,code)  { #name, code }
 #define CURSES_NAME(upper) { #upper, KEY_ ## upper }
-#define COUNT_CURSES  sizeof(curses_names)/sizeof(curses_names[0])
+#define COUNT_CURSES  TableSize(curses_names)
 static const CODENAME curses_names[] =
 {
     ASCII_NAME(ESC, '\033'),
@@ -419,7 +423,7 @@ static const CODENAME curses_names[] =
 };
 
 #define DIALOG_NAME(upper) { #upper, DLGK_ ## upper }
-#define COUNT_DIALOG  sizeof(dialog_names)/sizeof(dialog_names[0])
+#define COUNT_DIALOG  TableSize(dialog_names)
 static const CODENAME dialog_names[] =
 {
     DIALOG_NAME(OK),
@@ -455,9 +459,29 @@ static const CODENAME dialog_names[] =
     DIALOG_NAME(FINAL),
     DIALOG_NAME(SELECT),
     DIALOG_NAME(HELPFILE),
-    DIALOG_NAME(TRACE)
+    DIALOG_NAME(TRACE),
+    DIALOG_NAME(TOGGLE)
 };
 
+#define MAP2(letter,actual) { letter, actual }
+
+static const struct {
+    int letter;
+    int actual;
+} escaped_letters[] = {
+
+    MAP2('a', DLG_CTRL('G')),
+	MAP2('b', DLG_CTRL('H')),
+	MAP2('f', DLG_CTRL('L')),
+	MAP2('n', DLG_CTRL('J')),
+	MAP2('r', DLG_CTRL('M')),
+	MAP2('s', CHR_SPACE),
+	MAP2('t', DLG_CTRL('I')),
+	MAP2('\\', '\\'),
+};
+
+#undef MAP2
+
 static char *
 skip_white(char *s)
 {
@@ -576,8 +600,52 @@ make_binding(char *widget, int curses_key, int is_function, int dialog_key)
     return result;
 }
 
+static int
+decode_escaped(char **string)
+{
+    unsigned n;
+    int result = 0;
+
+    if (IsOctal(**string)) {
+	int limit = 3;
+	while (limit-- > 0 && IsOctal(**string)) {
+	    int ch = (**string);
+	    *string += 1;
+	    result = (result << 3) | (ch - '0');
+	}
+    } else {
+	for (n = 0; n < TableSize(escaped_letters); ++n) {
+	    if (**string == escaped_letters[n].letter) {
+		*string += 1;
+		result = escaped_letters[n].actual;
+		break;
+	    }
+	}
+    }
+    return result;
+}
+
+static char *
+encode_escaped(int value)
+{
+    static char result[80];
+    unsigned n;
+    bool found = FALSE;
+    for (n = 0; n < TableSize(escaped_letters); ++n) {
+	if (value == escaped_letters[n].actual) {
+	    found = TRUE;
+	    sprintf(result, "%c", escaped_letters[n].letter);
+	    break;
+	}
+    }
+    if (!found) {
+	sprintf(result, "%03o", value & 0xff);
+    }
+    return result;
+}
+
 /*
- * Parse the parameters of the "bindkeys" configuration-file entry.  This
+ * Parse the parameters of the "bindkey" configuration-file entry.  This
  * expects widget name which may be "*", followed by curses key definition and
  * then dialog key definition.
  *
@@ -614,8 +682,8 @@ dlg_parse_bindkey(char *params)
 	while (*p != '\0' && curses_key < 0) {
 	    if (escaped) {
 		escaped = FALSE;
-		curses_key = *p;
-	    } else if (*p == '\\') {
+		curses_key = decode_escaped(&p);
+	    } else if (*p == CHR_BACKSLASH) {
 		escaped = TRUE;
 	    } else if (modified) {
 		if (*p == '?') {
@@ -712,8 +780,12 @@ dump_curses_key(FILE *fp, int curses_key)
 	fprintf(fp, "~%c", curses_key - 64);
     } else if (curses_key == 255) {
 	fprintf(fp, "~?");
+    } else if (curses_key > 32 &&
+	       curses_key < 127 &&
+	       curses_key != CHR_BACKSLASH) {
+	fprintf(fp, "%c", curses_key);
     } else {
-	fprintf(fp, "\\%c", curses_key);
+	fprintf(fp, "%c%s", CHR_BACKSLASH, encode_escaped(curses_key));
     }
 }
 
