@@ -1,9 +1,9 @@
 /*
- *  $Id: menubox.c,v 1.149 2015/01/25 23:53:43 tom Exp $
+ *  $Id: menubox.c,v 1.151 2016/08/28 14:00:16 tom Exp $
  *
  *  menubox.c -- implements the menu box
  *
- *  Copyright 2000-2013,2015	Thomas E. Dickey
+ *  Copyright 2000-2015,2016	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public Licens, version 2.1e
@@ -304,7 +304,7 @@ dlg_menu(const char *title,
     static DLG_KEYS_BINDING binding[] = {
 	HELPKEY_BINDINGS,
 	ENTERKEY_BINDINGS,
-	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	' ' ),
+	TOGGLEKEY_BINDINGS,
 	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	KEY_RIGHT ),
 	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	TAB ),
 	DLG_KEYS_DATA( DLGK_FIELD_PREV,	KEY_BTAB ),
@@ -644,6 +644,7 @@ dlg_menu(const char *title,
 		dlg_draw_buttons(dialog, height - 2, 0, buttons, button,
 				 FALSE, width);
 		break;
+	    case DLGK_TOGGLE:
 	    case DLGK_ENTER:
 		if (is_inputmenu)
 		    result = dlg_ok_buttoncode(button);
