@@ -1,9 +1,9 @@
 /*
- *  $Id: textbox.c,v 1.110 2012/12/01 01:48:08 tom Exp $
+ *  $Id: textbox.c,v 1.113 2017/01/31 00:22:14 tom Exp $
  *
  *  textbox.c -- implements the text box
  *
- *  Copyright 2000-2011,2012	Thomas E.  Dickey
+ *  Copyright 2000-2016,2017	Thomas E.  Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -103,7 +103,7 @@ lseek_cur(MY_OBJ * obj, long offset)
     long actual = lseek_obj(obj, offset, SEEK_CUR);
 
     if (actual != offset) {
-	dlg_trace_msg("Lseek returned %ld, expected %ld\n", actual, offset);
+	DLG_TRACE(("# Lseek returned %ld, expected %ld\n", actual, offset));
     }
 }
 
@@ -679,7 +679,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 	DLG_KEYS_DATA( DLGK_PAGE_LAST,  'G' ),
 	DLG_KEYS_DATA( DLGK_PAGE_LAST,  KEY_END ),
 	DLG_KEYS_DATA( DLGK_PAGE_LAST,  KEY_LL ),
-	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  ' ' ),
+	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  CHR_SPACE ),
 	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  KEY_NPAGE ),
 	DLG_KEYS_DATA( DLGK_PAGE_PREV,  'B' ),
 	DLG_KEYS_DATA( DLGK_PAGE_PREV,  'b' ),
