@@ -1,9 +1,9 @@
 /*
- * $Id: timebox.c,v 1.49 2011/10/20 23:34:17 tom Exp $
+ * $Id: timebox.c,v 1.56 2016/08/28 14:16:33 tom Exp $
  *
  *  timebox.c -- implements the timebox dialog
  *
- *  Copyright 2001-2010,2011   Thomas E. Dickey
+ *  Copyright 2001-2013,2016   Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -41,8 +41,6 @@ typedef enum {
 
 struct _box;
 
-typedef int (*BOX_DRAW) (struct _box *, struct tm *);
-
 typedef struct _box {
     WINDOW *parent;
     WINDOW *window;
@@ -150,7 +148,7 @@ dialog_timebox(const char *title,
 	DLG_KEYS_DATA( DLGK_DELETE_RIGHT,KEY_DC ),
 	HELPKEY_BINDINGS,
 	ENTERKEY_BINDINGS,
-	DLG_KEYS_DATA( DLGK_ENTER,	' ' ),
+	TOGGLEKEY_BINDINGS,
 	DLG_KEYS_DATA( DLGK_FIELD_FIRST,KEY_HOME ),
 	DLG_KEYS_DATA( DLGK_FIELD_LAST, KEY_END ),
 	DLG_KEYS_DATA( DLGK_FIELD_LAST, KEY_LL ),
@@ -184,7 +182,7 @@ dialog_timebox(const char *title,
     WINDOW *dialog;
     time_t now_time = time((time_t *) 0);
     struct tm current;
-    int state = dlg_defaultno_button();
+    int state = dlg_default_button();
     const char **buttons = dlg_ok_labels();
     char *prompt = dlg_strclone(subtitle);
     char buffer[MAX_LEN];
@@ -296,12 +294,6 @@ dialog_timebox(const char *title,
 	    /* handle function-keys */
 	    if (fkey) {
 		switch (key) {
-		case DLGK_MOUSE(0):
-		    result = DLG_EXIT_OK;
-		    break;
-		case DLGK_MOUSE(1):
-		    result = DLG_EXIT_CANCEL;
-		    break;
 		case DLGK_MOUSE('H'):
 		    state = sHR;
 		    break;
@@ -311,8 +303,9 @@ dialog_timebox(const char *title,
 		case DLGK_MOUSE('S'):
 		    state = sSC;
 		    break;
+		case DLGK_TOGGLE:
 		case DLGK_ENTER:
-		    result = button;
+		    result = dlg_ok_buttoncode(button);
 		    break;
 		case DLGK_FIELD_PREV:
 		    state = dlg_prev_ok_buttonindex(state, sHR);
@@ -362,7 +355,11 @@ dialog_timebox(const char *title,
 		    goto retry;
 #endif
 		default:
-		    if (obj != 0) {
+		    if (is_DLGK_MOUSE(key)) {
+			result = dlg_ok_buttoncode(key - M_EVENT);
+			if (result < 0)
+			    result = DLG_EXIT_OK;
+		    } else if (obj != 0) {
 			int step = next_or_previous(key);
 			if (step != 0) {
 			    obj->value += step;
@@ -416,6 +413,7 @@ dialog_timebox(const char *title,
 
     dlg_add_result(buffer);
     dlg_add_separator();
+    dlg_add_last_key(-1);
 
     return CleanupResult(result, dialog, prompt, &save_vars);
 }
